/**
 * this will parse the conf file for adding or removing
 * diagnostic tools.
 *
 * Copyright (C) 2003 International Business Machines, Inc.
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation version 2 of the License.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *	General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License along
 *	with this program; if not, write to the Free Software Foundation, Inc.,
 *	675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * 	need api in libsysfs to get one attribute of one particular device.
 *	also an api to get all attributes of one device.
 *
 *
 *
 */

#include "sysdiag.h"


/**
 * this checks the section name that we are building the
 * diag info for is present in /sys/class/.
 *
 */
int check_section(char *name)
{
	char *path = NULL, *typename = NULL;
	struct sysfs_directory *current = NULL, *typedir = NULL;

	path = (char *)calloc(SYSFS_PATH_MAX, sizeof(char));

	if ((sysfs_get_mnt_path(path, SYSFS_PATH_MAX) != 0)) {
		perror("sysfs_get_mnt_path");
		fprintf(stderr, "Error getting sysfs mount path - exiting\n");
		exit(1);
	}
	strcat(path, SYSFS_CLASS_DIR);
	typedir = sysfs_open_directory(path);
	if (typedir == NULL) {
		fprintf(stderr, "Error opening sysfs bus directory at %s\n",
			path);
		return 1;
	}
	if ((sysfs_read_directory(typedir)) != 0) {
		fprintf(stderr, "Error reading sysfs bus directory at %s\n",
			path);
		return 1;
	}

	current = typedir->subdirs;
	while (current != NULL) {
		if (current->path != NULL) {
			typename = strrchr(current->path, '/');
			typename++;
		}
		if (strncasecmp(typename, name, strlen(name)) == 0) {
			memset(name,0,strlen(typename));
			strncpy(name, typename, strlen(typename));
			sysfs_close_directory(typedir);
			free(path);
			return 0;
		}
		current = current->next;
	}
	free(path);
	sysfs_close_directory(typedir);
	return 1;
}


int free_diag(struct diags *diag)
{
	struct diags *next = NULL;

	for (; diag != NULL; diag = next) {
		next = diag->next;
		free(diag->name);
		free(diag->inq);
		free(diag->test);
		free(diag->stat);
		free(diag);
	}
	return 0;

}


/**
 * We have individual sections, we need to get the diag strings
 * from them now.
 *
 */

struct diags * parse_diags(char *buf, struct diags *diag)
{
	
	char *ptr1 = NULL, *ptr2 = NULL;
	int size;

	if ((ptr1 = strstr(buf, "STAT=")) != NULL) {
		ptr1 += 5;
		if ((ptr2 = strchr(ptr1, '\n')) == NULL) {
			ptr2 = strchr(ptr1, '\0');
		}
		ptr2--;
		size = ptr2 - ptr1;
		size++;
		diag->stat = (char *)calloc(size+1, sizeof(char));
		strncpy(diag->stat, ptr1, size);
	}
	if ((ptr1 = strstr(buf,"INQ=")) != NULL) {
		ptr1 += 4;
		if ((ptr2 = strchr(ptr1, '\n')) == NULL) {
			ptr2 = strchr(ptr1, '\0');
		}
		ptr2--;
		size = ptr2 - ptr1;
		size++;
		diag->inq = (char *)calloc(size+1, sizeof(char));
		strncpy(diag->inq, ptr1, size);
	} 
	if ((ptr1 = strstr(buf, "TEST=")) != NULL) {
		ptr1 += 5;
		if ((ptr2 = strchr(ptr1, '\n')) == NULL) {
			ptr2 = strchr(ptr1, '\0');
		}
		ptr2--;
		size = ptr2 - ptr1;
		size++;
		diag->test = (char *)calloc(size+1, sizeof(char));
		strncpy(diag->test, ptr1, size);
	}
	return diag;
}


int free_sec(struct section *sec)
{
	struct section *sec_next = NULL;

	for (; sec != NULL; sec = sec_next) {
		sec_next = sec->next;
		free(sec->buf);
		free(sec->name);
		free(sec);
	}
	return 0;

}

/**
 * look for first section open '[' and the next section open
 * '[' between these lie conf for current class. If there is no
 * second '[' then we look for the end of file char '\0'.
 *
 * compare name of section with the classes that are there.
 * create link list of sections like.
 *
 * struct section {
 * 	struct section *next;
 * 	char *name;
 * 	char *buf;
 * }
 * 
 * parse the conf file into section and create a link list.
 * return the head of link list.
 * 
 * Parse each section, and fill the relavent variables for each
 * diagnostic type.
 * 
 */

struct diags * parse_file(char *buf, struct diags *diag_head)
{
	struct section *sec = NULL, *snext = NULL;
	char *name=NULL, *sbuf = NULL;
	char *opensec = NULL, *closesec = NULL;
	int size = 0;
	int last = 0;
	struct diags *diag_next = NULL;
	opensec = closesec = buf;
	while (opensec !=NULL ) {
		
		if ((opensec = strchr(closesec, '[')) == NULL) {
			break;
		}
		
		opensec++;
		if ((closesec = strchr(opensec, ']')) == NULL) {
			break;
		}
		size = 0;
		size = closesec - opensec;
		name = (char *)calloc(size+1, sizeof(char));
		strncpy(name, opensec, size);

		if (check_section(name) != 0) {
			closesec++;
			opensec = closesec;
			free(name);
			continue;
		}
		sec = malloc(sizeof(struct section));
		sec->buf = NULL;
		sec->name = name;
		closesec = strchr(closesec, '\n');
		closesec++;
		opensec = strchr(closesec, '[');
		if (opensec == NULL) {
			opensec = strchr(closesec, '\0');
			last=1;
		}
		
		size = 0;
		size = opensec - closesec;
		sbuf = (char *)calloc(size, sizeof(char));
		memcpy(sbuf, closesec, size);
		sec->buf = sbuf;
		sec->next = snext;
		snext = sec;
		closesec = opensec;
		if (last)
			break;	
	}
	
	for ( snext=sec; snext!=NULL; snext=snext->next) {
		diag_next = diag_head;
		diag_head = malloc(sizeof(struct diags));
		diag_head->test = NULL;
		diag_head->inq  = NULL;
		diag_head->stat = NULL;
		diag_head->name = NULL;
		
		if ((diag_head = (struct diags *)parse_diags(snext->buf,
						diag_head)) == NULL) {
			fprintf(stderr, "error parsing diags\n");
			exit(1);
		}
		size = strlen(snext->name);
		size++;
		diag_head->name = (char *)calloc(size, sizeof(char));
		strncpy(diag_head->name, snext->name, size);
		diag_head->next = diag_next;

	}
	free_sec(sec);
	return diag_head;

}


struct diags * parse_main(struct diags *diag)
{
	int fd,size;
	char *fbuf = NULL;
	struct stat buf;

	if (stat(CONF_FILE, &buf) == -1) {
		fprintf(stderr, "failed getting information about file %s\n", 
				CONF_FILE); 
		exit(1);
	}
	fd = open(CONF_FILE, O_RDONLY);
	if (fd == -1) {
		fprintf(stderr, "failed opening file %s\n", CONF_FILE);
		exit(1);
	}

	size = buf.st_size;
	fbuf = (char *)calloc(size,sizeof(char));

	if (read(fd,fbuf,size) == -1) {
		fprintf(stderr, "failed reading file %s\n", CONF_FILE);
		close(fd);
		free(fbuf);
		exit(1);
	}
	close(fd);
	diag = (struct diags *)parse_file(fbuf, diag);
	free(fbuf);
	return (struct diags *)diag;

}
